#!/usr/bin/env node


'use strict';

var _marked = [prompt].map(regeneratorRuntime.mark);

var yargs = require('yargs');
var chalk = require('chalk');

var PluginManager = require('./plugin.js');
var pe;
var showLogo = require('./logo.js');
var pkg = require('../package.json');
var co = require('co');
var log = require('./log.js');
var inquirer = require('inquirer');
var fullname = require('fullname');
var _ = require('lodash');
var autocomplete = require('inquirer-autocomplete-prompt');

inquirer.registerPrompt('autocomplete', autocomplete);

var argv = yargs.options({
    v: {
        alias: 'version',
        demand: false,
        describe: chalk.gray('output the version number')
    }
    // yeoman: {
    //     alias: 'y',
    //     type: 'boolean',
    //     describe: chalk.gray('use yeoman generator')
    // }
}).usage(chalk.bold('\nUsage:') + ' $0 ' + chalk.blue('[plugin]')).help('h').alias('h', 'help').describe('help', chalk.gray('show help infomation')).locale('en').argv;

if (argv.version || argv.v) {
    showLogo(pkg.version);
}

pe = new PluginManager(argv.yeoman);

co(regeneratorRuntime.mark(function _callee() {
    return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
            switch (_context.prev = _context.next) {
                case 0:
                    if (!(argv._.length > 0)) {
                        _context.next = 5;
                        break;
                    }

                    _context.next = 3;
                    return pe.run(argv._[0]);

                case 3:
                    _context.next = 6;
                    break;

                case 5:
                    return _context.delegateYield(prompt(), 't0', 6);

                case 6:
                case 'end':
                    return _context.stop();
            }
        }
    }, _callee, this);
}));

function prompt() {
    var name, installedPlugins, answers, runAnswers, choices, installAnswer, ret, updateAnswer;
    return regeneratorRuntime.wrap(function prompt$(_context2) {
        while (1) {
            switch (_context2.prev = _context2.next) {
                case 0:
                    _context2.next = 2;
                    return fullname();

                case 2:
                    name = _context2.sent;
                    installedPlugins = pe.getAllInstalled();
                    _context2.prev = 4;
                    _context2.next = 7;
                    return inquirer.prompt([{
                        type: 'list',
                        name: 'type',
                        message: 'Hello! ' + (name || '') + ' \u9009\u62E9\u60F3\u6267\u884C\u7684\u64CD\u4F5C\uFF1A',
                        choices: _.flatten([{
                            name: '运行脚手架',
                            value: 'run'
                        }, {
                            name: '安装脚手架',
                            value: 'install'
                        }, {
                            name: '更新脚手架',
                            value: 'update'
                        }, {
                            name: '退出',
                            value: 'exit'
                        }])
                    }]);

                case 7:
                    answers = _context2.sent;
                    _context2.next = 13;
                    break;

                case 10:
                    _context2.prev = 10;
                    _context2.t0 = _context2['catch'](4);

                    console.log(_context2.t0);

                case 13:
                    if (!(answers.type == 'run')) {
                        _context2.next = 23;
                        break;
                    }

                    if (!installedPlugins.length) {
                        _context2.next = 21;
                        break;
                    }

                    _context2.next = 17;
                    return inquirer.prompt([{
                        type: 'list',
                        name: 'plugin',
                        message: '选择要运行的插件：',
                        choices: installedPlugins.map(function (pl) {
                            return {
                                name: '   ' + pe.prefix + '-' + pl,
                                value: pl
                            };
                        })
                    }]);

                case 17:
                    runAnswers = _context2.sent;
                    return _context2.abrupt('return', pe.run(runAnswers.plugin));

                case 21:
                    log.info('没有可用的插件');
                    process.exit(0);

                case 23:
                    if (!(answers.type == 'exit')) {
                        _context2.next = 25;
                        break;
                    }

                    return _context2.abrupt('return');

                case 25:
                    if (!(answers.type == 'install')) {
                        _context2.next = 49;
                        break;
                    }

                    _context2.next = 28;
                    return pe.getAllNpm();

                case 28:
                    choices = _context2.sent;

                    if (!(choices.length >= 10)) {
                        _context2.next = 35;
                        break;
                    }

                    _context2.next = 32;
                    return inquirer.prompt([{
                        type: 'autocomplete',
                        message: '输入要安装的插件',
                        name: 'plugin',
                        source: function source(answers, input) {

                            return new Promise(function (resolve, reject) {
                                setTimeout(function () {
                                    resolve(choices.filter(function (v) {
                                        return new RegExp('^' + input, 'i').exec(v.name) != null;
                                    }));
                                }, 50);
                            });
                        }
                    }]);

                case 32:
                    installAnswer = _context2.sent;
                    _context2.next = 44;
                    break;

                case 35:
                    if (!(choices.length > 0)) {
                        _context2.next = 42;
                        break;
                    }

                    choices = choices.map(function (v) {
                        return {
                            name: pe.prefix + '-' + v.name + ' -- ' + chalk.gray(v.desc),
                            value: v.name
                        };
                    });
                    _context2.next = 39;
                    return inquirer.prompt([{
                        type: 'list',
                        message: '选择要安装的插件',
                        name: 'plugin',
                        choices: choices
                    }]);

                case 39:
                    installAnswer = _context2.sent;
                    _context2.next = 44;
                    break;

                case 42:
                    log.info('没有可用的插件');
                    process.exit(0);

                case 44:
                    _context2.next = 46;
                    return pe.install(installAnswer.plugin);

                case 46:
                    ret = _context2.sent;

                    if (!ret) {
                        _context2.next = 49;
                        break;
                    }

                    return _context2.delegateYield(prompt(), 't1', 49);

                case 49:
                    if (!(answers.type == 'update')) {
                        _context2.next = 63;
                        break;
                    }

                    if (!installedPlugins.length) {
                        _context2.next = 61;
                        break;
                    }

                    _context2.next = 53;
                    return inquirer.prompt([{
                        type: 'list',
                        message: '选择要更新的插件',
                        name: 'plugin',
                        choices: installedPlugins.map(function (v) {
                            return {
                                name: pe.prefix + '-' + v,
                                value: v
                            };
                        })
                    }]);

                case 53:
                    updateAnswer = _context2.sent;
                    _context2.next = 56;
                    return pe.update(updateAnswer.plugin);

                case 56:
                    ret = _context2.sent;

                    if (!ret) {
                        _context2.next = 59;
                        break;
                    }

                    return _context2.delegateYield(prompt(), 't2', 59);

                case 59:
                    _context2.next = 63;
                    break;

                case 61:

                    log.info('没有可用的插件');
                    process.exit(0);

                case 63:
                case 'end':
                    return _context2.stop();
            }
        }
    }, _marked[0], this, [[4, 10]]);
};