'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PluginExtend = require('plugin-extend');
var npmKeyWords = require('npm-keyword');
var request = require('superagent');

var log = require('./log.js');
var spawn = require('cross-spawn');
var LoadingBar = require('./bottomBar.js');
var Scaffold = require('./scaffold.js');

var PluginManager = function (_PluginExtend) {
    _inherits(PluginManager, _PluginExtend);

    function PluginManager(isYeoman) {
        _classCallCheck(this, PluginManager);

        var _this = _possibleConstructorReturn(this, (PluginManager.__proto__ || Object.getPrototypeOf(PluginManager)).call(this, {
            prefix: isYeoman ? 'generator' : 'bibo-scaffold'
        }));

        _this.entry = 'app'; //isYeoman ? 'app' : 'app';
        _this.keyword = isYeoman ? 'yeoman-generator' : 'bibo-scaffold';
        return _this;
    }

    _createClass(PluginManager, [{
        key: 'getAllInstalled',
        value: function getAllInstalled() {
            var _this2 = this;

            var sources = [];

            this.lookup().forEach(function (v) {
                var strs = v.split(':'),
                    key = strs[1],
                    source = strs[0];

                if (sources.indexOf(source) == -1 && key == _this2.entry) {
                    sources.push(source);
                }
            });

            return sources;
        }
    }, {
        key: 'getAllNpm',
        value: function getAllNpm() {
            var keyword = this.keyword,
                prefix = this.prefix,
                pBar = new LoadingBar('Searching');

            pBar.start();

            return new Promise(function (resolve, reject) {
                var pks = [];

                npmKeyWords(keyword).then(function (packages) {
                    packages.forEach(function (v) {
                        pks.push({
                            name: v.name.replace(prefix + '-', ''),
                            desc: v.description
                        });
                    });
                    pBar.stop();

                    resolve(pks);
                });
            });
        }
    }, {
        key: 'get',
        value: function get(name) {
            if (!name) {
                return;
            }
            return this.getPlugin(name + ':' + this.entry);
        }
    }, {
        key: 'exists',
        value: function exists(name) {
            var _this3 = this;

            var pName = void 0,
                latestUrl = void 0;

            return new Promise(function (resolve, reject) {
                if (!name) {
                    resolve(false);
                } else {
                    pName = _this3.prefix + '-' + name;
                    latestUrl = 'http://registry.npmjs.com/' + pName + '/latest';
                    request.get(latestUrl).timeout(3000).end(function (err, ret) {
                        var body = ret.body || {};

                        if (!err) {
                            resolve(!('error' in body));
                        } else {
                            resolve(false);
                        }
                    });
                }
            });
        }
    }, {
        key: 'run',
        value: function run(name, isCore) {
            var plugin = !isCore ? this.get(name) : {
                resolved: name
            },
                self = this,
                scaffold;

            return new Promise(function (resolve, reject) {

                if (plugin) {
                    scaffold = new Scaffold(plugin.resolved);
                    scaffold.run();
                } else {
                    var pName = self.prefix + '-' + name;

                    log.error('You don\u2019t seem to have a "' + pName + '" installed');
                    resolve(false);
                }
            });
        }
    }, {
        key: 'execNpm',
        value: function execNpm(name, type) {
            var pkgName = this.prefix + '-' + name;

            return new Promise(function (resolve) {
                spawn('npm', [type, '-g', pkgName], {
                    stdio: 'inherit'
                }).on('error', function (err) {
                    resolve(false);
                }).on('close', function () {
                    resolve(true);
                });
            });
        }
    }, {
        key: 'install',
        value: function install(name) {
            return this.execNpm(name, 'install');
        }
    }, {
        key: 'update',
        value: function update(name) {
            return this.execNpm(name, 'update');
        }
    }]);

    return PluginManager;
}(PluginExtend);

module.exports = PluginManager;