'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var co = require('co');
var _ = require('lodash');
var log = require('./log');
var path = require('path');
var pathIsAbsolute = require('path-is-absolute');

var vfs = require('vinyl-fs');
var through = require('through2');
var inquirer = require('inquirer');
var chalk = require('chalk');

function createInquirer(opts) {
    return inquirer.prompt(opts);
}

function parsePath(pth) {
    var extname = path.extname(pth);

    return {
        dirname: path.dirname(pth),
        basename: path.basename(pth, extname),
        extname: extname
    };
}

var Scaffold = function () {
    function Scaffold(resolved) {
        _classCallCheck(this, Scaffold);

        this.resolved = resolved;

        this.opts = require(this.resolved)(log);
        this.sourceRoot(path.join(path.dirname(this.resolved), '../templates'));
        this.destRoot(process.cwd());
    }

    _createClass(Scaffold, [{
        key: 'getModel',
        value: function getModel() {
            var _this = this;

            var ret,
                model,
                questions = [];

            return new Promise(function (resolve) {
                if (_this.data) {
                    resolve(_this.data);
                } else {
                    model = _this.opts.model;
                    if (_.isFunction(model)) {
                        ret = model.call(_this);
                        if (ret instanceof Promise) {
                            ret.then(function (data) {
                                _this.data = data;
                                resolve(data);
                            });
                        } else {
                            _this.data = ret;
                            resolve(ret);
                        }
                    } else if (_.isPlainObject(model)) {
                        Object.keys(model).forEach(function (key) {
                            questions.push(Object.assign({
                                name: key,
                                type: 'input',
                                validate: function validate(input) {
                                    if (_.trim(input)) {
                                        return true;
                                    }
                                    return '必填项';
                                }
                            }, model[key]));
                        });

                        if (questions.length) {
                            createInquirer(questions).then(function (answers) {
                                _this.data = answers;
                                resolve(answers);
                            });
                        }
                    } else {
                        log.error('The model property of the plug-in must return a function or object');
                        process.exit(0);
                    }
                }
            });
        }
    }, {
        key: 'sourceRoot',
        value: function sourceRoot(rootPath) {
            if (_.isString(rootPath)) {
                this._sourceRoot = path.resolve(rootPath);
            }

            return this._sourceRoot;
        }
    }, {
        key: 'destRoot',
        value: function destRoot(rootPath) {
            if (_.isString(rootPath)) {
                this._destRoot = path.resolve(rootPath);
            }

            return this._destRoot;
        }
    }, {
        key: 'tmplPath',
        value: function tmplPath() {
            var filepath = path.join.apply(path, arguments);

            if (!pathIsAbsolute(filepath)) {
                filepath = path.join(this.sourceRoot(), filepath);
            }

            return filepath;
        }
    }, {
        key: 'destPath',
        value: function destPath() {
            var filepath = path.join.apply(path, arguments);

            if (!pathIsAbsolute(filepath)) {
                filepath = path.join(this.destRoot(), filepath);
            }

            return filepath;
        }
    }, {
        key: '_write',
        value: function _write(glob, dest, vars, isRename) {
            var self = this,
                paths = [],
                opts;

            dest = dest || this.destRoot();

            vars = vars || {};

            opts = {
                cwd: self.sourceRoot(),
                cwdbase: true,
                dot: true
            };
            return new Promise(function (resolve, reject) {
                var stream = vfs.src(glob, opts).pipe(function (vars) {
                    return through.obj(function (file, enc, cb) {
                        if (!file.stat.isFile()) {
                            return cb();
                        }
                        var contents = _.template(file.contents.toString())(vars);

                        file.contents = new Buffer(contents);
                        if (isRename) {
                            var pPth = parsePath(dest);

                            file.path = path.join(parsePath(file.path).dirname, pPth.basename + pPth.extname);
                            log.write(chalk.green('   create ' + path.relative(process.cwd(), dest) + '\r'));
                        } else {
                            log.write(chalk.green('   create ' + path.join(path.relative(process.cwd(), self.destRoot()), file.relative) + '\r'));
                        }
                        this.push(file);

                        cb();
                    });
                }(vars)).pipe(vfs.dest(isRename ? path.dirname(dest) : dest));

                stream.on('finish', function () {

                    //log.write(_.flatten(paths).join(''));
                    resolve();
                }).on('error', function () {
                    reject();
                });
            });
        }
    }, {
        key: 'run',
        value: function run() {
            var _this2 = this;

            return new Promise(function (resolve) {
                var opts = _this2.opts,
                    flag = false;

                _this2.getModel().then(function (data) {
                    if (_.isFunction(opts.write)) {
                        opts.write.call(_this2, data);
                        flag = true;
                    } else if (_.isPlainObject(opts.write)) {
                        Object.keys(opts.write).forEach(function (key) {
                            if (_.isFunction(opts.write[key])) {
                                flag = true;
                                opts.write[key].call(_this2, data);
                            }
                        });
                    }
                    if (!flag) {

                        _this2._write('**/*', null, data);
                        resolve();
                    } else {
                        resolve();
                    }
                });
            });
        }
    }, {
        key: 'copyTpl',
        value: function copyTpl(src, to) {
            var _this3 = this;

            return new Promise(function (resolve) {
                if (!pathIsAbsolute(src)) {
                    src = _this3.tmplPath(src);
                }
                if (!pathIsAbsolute(to)) {
                    to = _this3.destPath(to);
                }
                _this3.getModel().then(function (data) {
                    _this3._write(src, to, data, true);
                    resolve();
                });
            });
        }
    }]);

    return Scaffold;
}();

module.exports = Scaffold;